package field

import (
	"context"
	"fmt"
	"reflect"
	"strings"
)

var stringType = &StringType{}

type StringParameters struct{}

func (s StringParameters) Type() Type                                              { return stringType }
func (s *StringParameters) Clone(reset bool) Parameters                            { return s }
func (s StringParameters) GetField(f *Field, name string) *Field                   { return nil }
func (s StringParameters) ListFields(f *Field, filter ...FieldFilterFunc) []*Field { return nil }

type StringType struct{}

func (s StringType) Name() string {
	return "string"
}

func (StringType) NewParameters() Parameters {
	return &StringParameters{}
}

func (StringType) IsEmpty(v interface{}) bool {
	s, _ := v.(string)
	return s == ""
}

func (StringType) Decode(_ context.Context, _ *Field, v interface{}) (interface{}, error) {
	if v == nil {
		return nil, nil
	}
	if _, ok := v.(string); ok {
		return v, nil
	}
	t := reflect.ValueOf(v)
	if t.Kind() == reflect.String {
		return t.String(), nil
	}
	return nil, fmt.Errorf("StringField decode error: unsupported value type : \"%s\"", reflect.ValueOf(v).Kind())
}

func (StringType) Encode(_ context.Context, _ *Field, v interface{}) (interface{}, error) {
	if v == nil {
		return nil, nil
	}
	if _, ok := v.(string); ok {
		return v, nil
	}
	t := reflect.ValueOf(v)
	if t.Kind() == reflect.String {
		return t.String(), nil
	}
	return nil, fmt.Errorf("StringField encode error: unsupported value type : \"%s\"", reflect.ValueOf(v).Kind())
}

func (StringType) NonStrictConvert(_ context.Context, _ *Field, v interface{}) interface{} {
	if v == nil {
		return nil
	}

	switch v := v.(type) {
	case string:
		return v
	case []interface{}:
		// Преобразование списка в строку происходит через пробел,
		// чтобы сохранить совместимость при обратном преобразовании в список
		elems := make([]string, len(v))
		for i, elem := range v {
			elems[i] = fmt.Sprintf("%v", elem)
		}
		return strings.Join(elems, " ")
	}

	// По умолчанию используется стандартное форматирование
	return fmt.Sprintf("%v", v)
}

func String(o ...interface{}) *Field {
	return NewField(&StringParameters{}, o...)
}
